@@ -107,19 +107,22 @@ static CK_ATTRIBUTE public_key_filter[] = {
 static SECURITY_STATUS NCryptP11StorageProvider_dtor(NCRYPT_HANDLE handle)
 {
 	NCryptP11ProviderHandle* provider = (NCryptP11ProviderHandle*)handle;
-	CK_RV rv = 0;
+	CK_RV rv = CKR_OK;
 
-	WINPR_ASSERT(provider);
-	rv = provider->p11->C_Finalize(NULL);
-	if (rv != CKR_OK)
+	if (provider)
 	{
+		if (provider->p11 && provider->p11->C_Finalize)
+			rv = provider->p11->C_Finalize(NULL);
+		if (rv != CKR_OK)
+		{
+		}
+
+		free(provider->modulePath);
+
+		if (provider->library)
+			FreeLibrary(provider->library);
 	}
 
-	free(provider->modulePath);
-
-	if (provider->library)
-		FreeLibrary(provider->library);
-
 	return winpr_NCryptDefault_dtor(handle);
 }
 
@@ -330,7 +333,7 @@ static const char* CK_RV_error_string(CK_RV rv)
 		ERR_ENTRY(CKR_MUTEX_NOT_LOCKED);
 		ERR_ENTRY(CKR_FUNCTION_REJECTED);
 		default:
-			snprintf(generic_buffer, sizeof(generic_buffer), "unknown 0x%lx", rv);
+			(void)snprintf(generic_buffer, sizeof(generic_buffer), "unknown 0x%lx", rv);
 			return generic_buffer;
 	}
 #undef ERR_ENTRY
@@ -530,7 +533,7 @@ static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 
 		if (dest)
 		{
-			memcpy(dest, r, retLen * 2);
+			memcpy(dest, r, sizeof(WCHAR) * retLen);
 			dest[retLen] = 0;
 		}
 	}
@@ -549,13 +552,13 @@ static void wprintKeyName(LPWSTR str, CK_SLOT_ID slotId, CK_BYTE* id, CK_ULONG i
 
 	bytePtr = ((CK_BYTE*)&slotId);
 	for (CK_ULONG i = 0; i < sizeof(slotId); i++, bytePtr++, ptr += 2)
-		snprintf(ptr, 3, "%.2x", *bytePtr);
+		(void)snprintf(ptr, 3, "%.2x", *bytePtr);
 
 	*ptr = '\\';
 	ptr++;
 
 	for (CK_ULONG i = 0; i < idLen; i++, id++, ptr += 2)
-		snprintf(ptr, 3, "%.2x", *id);
+		(void)snprintf(ptr, 3, "%.2x", *id);
 
 	ConvertUtf8NToWChar(asciiName, ARRAYSIZE(asciiName), str,
 	                    strnlen(asciiName, ARRAYSIZE(asciiName)) + 1);
@@ -795,7 +798,7 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 			    (1 + (sizeof(key->slotId) * 2) /*slotId*/ + 1 + (key->idLen * 2) + 1) * 2;
 
 			convertKeyType(key->keyType, NULL, 0, &algoSz);
-			KEYNAME_SZ += (algoSz + 1) * 2;
+			KEYNAME_SZ += (1ULL + algoSz) * 2ULL;
 
 			keyName = calloc(1, sizeof(*keyName) + KEYNAME_SZ);
 			if (!keyName)
@@ -903,10 +906,10 @@ static SECURITY_STATUS get_piv_container_name(NCryptP11KeyHandle* key, const BYT
 	p = Stream_Buffer(&s);
 
 	/* Construct the value Windows would use for a PIV key's container name */
-	snprintf(container_name, PIV_CONTAINER_NAME_LEN + 1,
-	         "%.2x%.2x%.2x%.2x-%.2x%.2x-%.2x%.2x-%.2x%.2x-%.2x%.2x%.2x%.2x%.2x%.2x", p[3], p[2],
-	         p[1], p[0], p[5], p[4], p[7], p[6], p[8], p[9], p[10], p[11], p[12], piv_tag[0],
-	         piv_tag[1], piv_tag[2]);
+	(void)snprintf(container_name, PIV_CONTAINER_NAME_LEN + 1,
+	               "%.2x%.2x%.2x%.2x-%.2x%.2x-%.2x%.2x-%.2x%.2x-%.2x%.2x%.2x%.2x%.2x%.2x", p[3],
+	               p[2], p[1], p[0], p[5], p[4], p[7], p[6], p[8], p[9], p[10], p[11], p[12],
+	               piv_tag[0], piv_tag[1], piv_tag[2]);
 
 	/* And convert it to UTF-16 */
 	union
@@ -1214,6 +1217,7 @@ static SECURITY_STATUS initialize_pkcs11(HANDLE handle,
 		goto fail;
 	}
 
+	WINPR_ASSERT(ret->p11);
 	WINPR_ASSERT(ret->p11->C_Initialize);
 	rv = ret->p11->C_Initialize(NULL);
 	if (rv != CKR_OK)
@@ -1245,12 +1249,13 @@ SECURITY_STATUS NCryptOpenP11StorageProviderEx(NCRYPT_PROV_HANDLE* phProvider,
 
 	while (*modulePaths)
 	{
-		HANDLE library = LoadLibrary(*modulePaths);
+		const char* modulePath = *modulePaths++;
+		HANDLE library = LoadLibrary(modulePath);
 		typedef CK_RV (*c_get_function_list_t)(CK_FUNCTION_LIST_PTR_PTR);
 		c_get_function_list_t c_get_function_list = NULL;
 		NCryptP11ProviderHandle* provider = NULL;
 
-		WLog_DBG(TAG, "Trying pkcs11 module '%s'", *modulePaths);
+		WLog_DBG(TAG, "Trying pkcs11 module '%s'", modulePath);
 		if (!library)
 		{
 			status = NTE_PROV_DLL_NOT_FOUND;
@@ -1272,15 +1277,19 @@ SECURITY_STATUS NCryptOpenP11StorageProviderEx(NCRYPT_PROV_HANDLE* phProvider,
 		}
 
 		provider = (NCryptP11ProviderHandle*)*phProvider;
-		provider->modulePath = _strdup(*modulePaths);
+		provider->modulePath = _strdup(modulePath);
+		if (!provider->modulePath)
+		{
+			status = NTE_NO_MEMORY;
+			goto out_load_library;
+		}
 
-		WLog_DBG(TAG, "module '%s' loaded", *modulePaths);
+		WLog_DBG(TAG, "module '%s' loaded", modulePath);
 		return ERROR_SUCCESS;
 
 	out_load_library:
 		if (library)
 			FreeLibrary(library);
-		modulePaths++;
 	}
 
 	return status;
